#include <SPI.h>
#include "image2.h"
#include "font8x16.h"

#define TFT_CS  9    // Data/Command
#define TFT_DC  8    // Data/Command
#define TFT_RST 12   // RESET
#define TFT_BL  13   // BACK LIGHT

#define TFT_WIDTH 160
#define TFT_HIGH  128
uint8_t SPIBuf[256] ;   // SPI転送用バッファ
uint8_t fontColorF[2] ; // TEXT FOR GROND COLOR
uint8_t fontColorB[2] ; // TEXT BACK GROND COLOR

char str[64] ;

SPISettings settings = SPISettings(32000000, MSBFIRST, SPI_MODE0);

// TFTにコマンドを送信
void tftSendCommand(uint8_t command) {
  digitalWrite(TFT_CS, LOW); // 
  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(command);
  digitalWrite(TFT_CS, HIGH);  //
}

// TFTにコマンド+1バイトデータを送信
void tftSendCommand1(uint8_t command, uint8_t data1) {
  digitalWrite(TFT_CS, LOW); // 
  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(command);
  digitalWrite(TFT_DC, HIGH); // Command mode
  SPI.transfer(data1);
  digitalWrite(TFT_CS, HIGH);  //
}

// TFTにコマンド+2バイトデータを送信
void tftSendCommand2(uint8_t command, uint8_t data1, uint8_t data2) {
  digitalWrite(TFT_CS, LOW); // 
  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(command);
  digitalWrite(TFT_DC, HIGH); // Command mode
  SPI.transfer(data1);
  SPI.transfer(data2);
  digitalWrite(TFT_CS, HIGH);  // 
}

// TFTにコマンド+3バイトデータを送信
void tftSendCommand3(uint8_t command, uint8_t data1, uint8_t data2, uint8_t data3) {
  digitalWrite(TFT_CS, LOW); // 
  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(command);
  digitalWrite(TFT_DC, HIGH); // Command mode
  SPI.transfer(data1);
  SPI.transfer(data2);
  SPI.transfer(data3);
  digitalWrite(TFT_CS, HIGH);  //
}

// TFTにコマンド+4バイトデータを送信
void tftSendCommand4(uint8_t command, uint8_t data1, uint8_t data2, uint8_t data3, uint8_t data4) {
  digitalWrite(TFT_CS, LOW); // 
  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(command);
  digitalWrite(TFT_DC, HIGH); // Command mode
  SPI.transfer(data1);
  SPI.transfer(data2);
  SPI.transfer(data3);
  SPI.transfer(data4);
  digitalWrite(TFT_CS, HIGH);  //
}

// 画面クリア
void cls(bool rot) {
  if (rot) {
    tftSendCommand4(0x2A,0,0,0,131) ; // Colmun Address
    tftSendCommand4(0x2B,0,0,0,161) ; // Row Address
  } else {
    tftSendCommand4(0x2A,0,0,0,160) ; // Colmun Address
    tftSendCommand4(0x2B,0,0,0,131) ; // Row Address
  }

  digitalWrite(TFT_CS, LOW);
  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(0x2C);
  digitalWrite(TFT_DC, HIGH); // Data mode
  for (int i=0;i<160;i++) {
    for (int j=0;j<131;j++) {
      SPI.transfer(B00000000);
      SPI.transfer(B00000000);
    }
  }
  digitalWrite(TFT_DC, LOW); // Command mode
  digitalWrite(TFT_CS, HIGH);  //
}

// 塗りつぶし
void fill(int xs,int ys , int xe,int ye , uint8_t c1 , uint8_t c2) {
  tftSendCommand4(0x2A,0,xs,0,xe) ; // Colmun Address
  tftSendCommand4(0x2B,0,ys,0,ye) ; // Row Address

  digitalWrite(TFT_CS, LOW);
  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(0x2C);
  digitalWrite(TFT_DC, HIGH); // Data mode
  for (int i=0;i<xe-xs+1;i++) {
    for (int j=0;j<ye-ys+1;j++) {
      uint8_t dw = c1 ;
      SPI.transfer(dw);
      dw = c2 ;
      SPI.transfer(dw);
    }
  }
  digitalWrite(TFT_DC, LOW); // Command mode
  digitalWrite(TFT_CS, HIGH);  //
}

// BITマップ表示
void drowBitMap(int offsetX , int offsetY , int width, int hight, uint8_t bitMap[]) {
  uint8_t endX = offsetX + width - 1;
  uint8_t endY = offsetY + hight - 1;
  tftSendCommand4(0x2A,0,offsetX,0,endX) ; // Colmun Address
  tftSendCommand4(0x2B,0,offsetY,0,endY) ; // Row Address
  delayMicroseconds(500);

  int pos = 0 ;
  digitalWrite(TFT_CS, LOW);
  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(0x2C);
  digitalWrite(TFT_DC, HIGH); // Data mode
  for (int row = 0 ; row < hight ; row ++) {
    // 表示データ転送
    // Arduinoだとメモリが足りなくて１画面分を一気に転送する事ができない
    // なので、1Lineずつ送信している
    for (int col = 0 ; col < width * 2 ; col ++ ) {
      SPIBuf[col] = bitMap[pos];
      //      SPIBuf[col] = 0xF8 ;
      pos ++ ;
    }
    SPI.transfer(SPIBuf, width * 2);
  }
  digitalWrite(TFT_DC, LOW); // Command mode
  digitalWrite(TFT_CS, HIGH);  //
}

void DrowFont(uint8_t code , uint8_t x, uint8_t y) {
  int pos = (code - 0x20) * 16 ;
  memset(SPIBuf,0,8*16*2) ;
  for (int i=0;i<16;i++) {
    uint8_t data = ssd1306xled_font8x16[pos + i];
    uint8_t bitMast = 0x01 ; 
    for (int j=0;j<8;j++) {
      int fpos = 0 ;
      if (i<8) {
        fpos = (j*8+i)*2 ;
      } else {
        fpos = ((j+8)*8+(i-8))*2 ;
      }
      if ((data & bitMast) == 0) {
        SPIBuf[fpos] = fontColorB[0] ;
        SPIBuf[fpos+1] = fontColorB[1] ;
      } else {
        SPIBuf[fpos] = fontColorF[0] ;
        SPIBuf[fpos+1] = fontColorF[1] ;
      }
      bitMast <<= 1 ;
    }
  }

  uint8_t endX = x + 7 ;
  uint8_t endY = y + 15;

  tftSendCommand4(0x2A,0,x,0,endX) ; // Colmun Address
  tftSendCommand4(0x2B,0,y,0,endY) ; // Row Address
  digitalWrite(TFT_CS, LOW);
  digitalWrite(TFT_DC, LOW); // Command mode
  SPI.transfer(0x2C);
  digitalWrite(TFT_DC, HIGH); // Data mode
  SPI.transfer(SPIBuf, 8*16*2);
  digitalWrite(TFT_DC, LOW); // Command mode
  digitalWrite(TFT_CS, HIGH);  // 
  delayMicroseconds(500);
}

void DrowStr(char str[] , uint8_t x , uint8_t y) {
  for (int i=0;str[i] != 0 ; i++) {
      DrowFont(str[i],x,y) ;
      x += 8 ;
      if (x+8>=96) {
        x = 0 ;
        y += 16 ;
        if (y + 16 >= 64) {
          y = 0 ; 
        }
      }
  }
}

void setup() {
  Serial.begin(9600);
  Serial.print(F("Pico LCD-1.8 (ST3375)Test"));

  SPI.begin();  //SPIを初期化、SCK、MOSI、SSの各ピンの動作は出力、SCK、MOSIはLOW、SSはHIGH
  SPI.beginTransaction(settings);

  // -----  ST3375 INITIAL -----
  pinMode(TFT_CS, OUTPUT);
  pinMode(TFT_DC, OUTPUT);
  pinMode(TFT_BL, OUTPUT);
  pinMode(TFT_RST, OUTPUT);

  // BACK LIGHT ON
  digitalWrite(TFT_BL, HIGH);

  // --- HARD Ware Reset
  digitalWrite(TFT_RST, HIGH);
  delay(500);                  // VDD goes high at start, pause for 500 ms
  digitalWrite(TFT_RST, LOW);  // Bring reset low
  delay(100);                  // Wait 100 ms
  digitalWrite(TFT_RST, HIGH); // Bring out of reset
  delay(500);                  // Wait 500 ms, more then 120 ms
  // --- SOFT Ware Reset
  tftSendCommand(0x01) ;       // SOFTWARE RESET
  delay(50);

  // --- Initial Comands
  bool r = false ;
  tftSendCommand(0x28) ;            // Display OFF : 0x29 - Display ON
  delay(500);

  tftSendCommand(0x11) ;            // Sleep Out
  delay(500);
  tftSendCommand1(0x3A,05) ;        // 16Bit Pixel Mode
  delay(10);
  tftSendCommand1(0x36,B10100000) ; r = false; // MX MY MV ML RGB MH x x:横向き１
//  tftSendCommand1(0x36,B01100000) ; r = false;  // MX MY MV ML RGB MH x x：横向き２
//  tftSendCommand1(0x36,B00000000) ; r = true;  // MX MY MV ML RGB MH x x：縦向き１
//  tftSendCommand1(0x36,B11000000) ; r = true;  // MX MY MV ML RGB MH x x：縦向き２(多分これがスタンダード)
  tftSendCommand2(0xB6,0x15,0x02) ; // Display settings #5
  tftSendCommand1(0xB4,0x00) ;      // Display inversion control

  tftSendCommand(0x13) ;            // NomalDisplayMode
  tftSendCommand(0x20) ;            // Display Inversion Off

  // ---
  cls(r) ;
  delay(500);
  tftSendCommand(0x29) ;            // Display ON : 0x28 - Display OFF
  delay(500);
}

// the loop function runs over and over again forever
void loop() {
  bool r = false ;
  fontColorB[0] = 0x00 ; fontColorB[1] = 0x00 ; // TEXT BACK GROND COLOR
  fontColorF[0] = 0xFF ; fontColorF[1] = 0xFF ; // TEXT FOR GROND COLOR - WHITE
  for (int i=0;i<4;i++) {
    switch(i) {
      case  0:
        sprintf(str,"B10100000");
        tftSendCommand1(0x36,B10100000) ;// MX MY MV ML RGB MH x x:横向き１
        r = false; 
        break ;
      case  1:
        sprintf(str,"B01100000");
        tftSendCommand1(0x36,B01100000) ; // MX MY MV ML RGB MH x x：横向き２
        r = false; 
        break ;
      case  2:
        sprintf(str,"B00000000");
        tftSendCommand1(0x36,B00000000) ; // MX MY MV ML RGB MH x x：縦向き１
        r = true; 
        break ;
      case  3:
        sprintf(str,"B11000000");
        tftSendCommand1(0x36,B11000000) ; // MX MY MV ML RGB MH x x：縦向き２(多分これがスタンダード)
        r = true; 
        break ;
    }
    cls(r) ;
    DrowStr(str,10,0) ;
    drowBitMap(10,20,96,64,bitmapData2) ;
    fill(10, 90,106, 99,B11111000,B00000000) ;
    fill(10,102,106,111,B00000111,B11100000) ;
    fill(10,113,106,122,B00000000,B00011111) ;
    delay(5000);
 }
}
